@@ -3,8 +3,8 @@
  *
  * pdfresurrect - PDF history extraction tool
  *
- * Copyright (C) 2008, 2009, 2010 Matt Davis (enferex) of 757Labs
- * (www.757labs.com)
+ * Copyright (C) 2008, 2009, 2010, 2012 Matt Davis (enferex) of 757Labs
+ * (www.757labs.org)
  *
  * pdf.c is part of pdfresurrect.
  * pdfresurrect is free software: you can redistribute it and/or modify
@@ -28,6 +28,28 @@
 #include "main.h"
 
 
+/* 
+ * Macros
+ */
+
+/* SAFE_F
+ *
+ * Safe file read: use for fgetc() calls, this is really ugly looking. 
+ * _fp:   FILE * handle
+ * _expr: The expression with fgetc() in it:
+ * 
+ *  example: If we get a character from the file and it is ascii character 'a'
+ *           This assumes the coder wants to store the 'a' in variable ch
+ *           Kinda pointless if you already know that you have 'a', but for
+ *           illustrative purposes.
+ *
+ *  if (SAFE_F(my_fp, ((c=fgetc(my_fp)) == 'a')))
+ *              do_way_cool_stuff();
+ */
+#define SAFE_F(_fp, _expr) \
+    ((!ferror(_fp) && !feof(_fp) && (_expr)))
+
+
 /*
  * Forwards
  */
@@ -166,6 +188,9 @@ int pdf_load_xrefs(FILE *fp, pdf_t *pdf)
       if (strstr(buf, "%%EOF"))
         ++pdf->n_xrefs;
 
+    if (!pdf->n_xrefs)
+      return 0;
+
     /* Load in the start/end positions */
     fseek(fp, 0, SEEK_SET);
     pdf->xrefs = calloc(1, sizeof(xref_t) * pdf->n_xrefs);
@@ -185,7 +210,7 @@ int pdf_load_xrefs(FILE *fp, pdf_t *pdf)
 
         /* Rewind until we find end of "startxref" */
         pos_count = 0;
-        while ((x = fgetc(fp)) != 'f')
+        while (SAFE_F(fp, ((x = fgetc(fp)) != 'f')))
           fseek(fp, pos - (++pos_count), SEEK_SET);
         
         /* Suck in end of "startxref" to start of %%EOF */
@@ -262,7 +287,7 @@ void pdf_load_pages_kids(FILE *fp, pdf_t *pdf)
         if (pdf->xrefs[i].version && (pdf->xrefs[i].end != 0))
         {
             fseek(fp, pdf->xrefs[i].start, SEEK_SET);
-            while (fgetc(fp) != 't')
+            while (SAFE_F(fp, (fgetc(fp) != 't')))
                 ; /* Iterate to trailer */
 
             /* Get root catalog */
@@ -376,6 +401,7 @@ void pdf_zero_object(
     for (i=0; i<obj_sz; i++)
       fputc('0', fp);
 
+    printf("Zeroed object %d\n", entry->obj_id);
     free(obj);
 }
 
@@ -415,7 +441,7 @@ void pdf_summarize(
 
     /* Count versions */
     n_versions = pdf->n_xrefs;
-    if (pdf->xrefs[0].is_linear)
+    if (n_versions && pdf->xrefs[0].is_linear)
       --n_versions;
 
     /* Ignore bad xref entry */
@@ -424,7 +450,7 @@ void pdf_summarize(
         --n_versions;
 
     /* If we have no valid versions but linear, count that */
-    if (!n_versions && pdf->xrefs[0].is_linear)
+    if (!pdf->n_xrefs || (!n_versions && pdf->xrefs[0].is_linear))
       n_versions = 1;
 
     /* Compare each object (if we dont have xref streams) */
@@ -528,7 +554,8 @@ int pdf_display_creator(const pdf_t *pdf, int xref_idx)
 
 
 /* Checks if the xref is valid and sets 'is_stream' flag if the xref is a
- * stream (PDF 1.5 or higher) */
+ * stream (PDF 1.5 or higher)
+ */
 static int is_valid_xref(FILE *fp, pdf_t *pdf, xref_t *xref)
 {
     int   is_valid;
@@ -584,7 +611,7 @@ static void load_xref_from_plaintext(FILE *fp, xref_t *xref)
     pos = xref->end;
     fseek(fp, pos, SEEK_SET);
     while (ftell(fp) != 0)
-      if (fgetc(fp) == '/' && fgetc(fp) == 'S')
+      if (SAFE_F(fp, (fgetc(fp) == '/' && fgetc(fp) == 'S')))
         break;
       else
         fseek(fp, --pos, SEEK_SET);
@@ -701,7 +728,7 @@ static void get_xref_linear_skipped(FILE *fp, xref_t *xref)
 
     /* If we found 'trailer' look backwards for 'xref' */
     ch = 0;
-    while (!ferror(fp) && ((ch = fgetc(fp)) != 'x'))
+    while (SAFE_F(fp, ((ch = fgetc(fp)) != 'x')))
       fseek(fp, -2, SEEK_CUR);
 
     if (ch == 'x')
@@ -793,24 +820,26 @@ static void load_creator(FILE *fp, pdf_t *pdf)
 
         /* Find trailer */
         fseek(fp, pdf->xrefs[i].start, SEEK_SET);
-        while (fgetc(fp) != 't')
+        while (SAFE_F(fp, (fgetc(fp) != 't')))
             ; /* Iterate to "trailer" */
 
         /* Look for "<< ....... /Info ......" */
-        while ((c = fgetc(fp)) != '>')
-          if ((c == '/') && (fgetc(fp) == 'I') && ((fgetc(fp) == 'n')))
+        c = '\0';
+        while (SAFE_F(fp, ((c = fgetc(fp)) != '>')))
+          if (SAFE_F(fp, ((c == '/') &&
+                          (fgetc(fp) == 'I') && ((fgetc(fp) == 'n')))))
             break;
 
         /* Could not find /Info in trailer */
         END_OF_TRAILER(c, start, fp);
 
-        while (!isspace(c = fgetc(fp)) && (c != '>'))
+        while (SAFE_F(fp, (!isspace(c = fgetc(fp)) && (c != '>'))))
             ; /* Iterate to first white space /Info<space><data> */
 
         /* No space between /Info and it's data */
         END_OF_TRAILER(c, start, fp);
 
-        while (isspace(c = fgetc(fp)) && (c != '>'))
+        while (SAFE_F(fp, (isspace(c = fgetc(fp)) && (c != '>'))))
             ; /* Iterate right on top of first non-whitespace /Info data */
 
         /* No data for /Info */
@@ -819,7 +848,7 @@ static void load_creator(FILE *fp, pdf_t *pdf)
         /* Get obj id as number */
         buf_idx = 0;
         obj_id_buf[buf_idx++] = c;
-        while (!isspace(c = fgetc(fp)) && (c != '>'))
+        while (SAFE_F(fp, (!isspace(c = fgetc(fp)) && (c != '>'))))
           obj_id_buf[buf_idx++] = c;
 
         END_OF_TRAILER(c, start, fp);
@@ -1222,7 +1251,7 @@ static char *get_header(FILE *fp)
     
     start = ftell(fp);
     fseek(fp, 0, SEEK_SET);
-    fread(header, 1024, 1, fp);
+    fread(header, 1023, 1, fp);
     fseek(fp, start, SEEK_SET);
     
     return header;
